#include "fs/devices/Console.h"
#include "config.h"
#include "render/TextRenderer.h"
#include "std/stdio.h"
#include "std/stdlib.h"
#include "std/string.h"

char* conin_buffer = nullptr;
uint64_t conin_bufsize = 0;

int ConsoleDevice::would_block(VFS::Node*)
{
    return conin_bufsize == 0;
}

extern uint64_t clock_now();

VFS::Node* ConsoleDevice::create_new(const char* devname)
{
    VFS::Node* dev = new VFS::Node;
    dev->write_func = ConsoleDevice::write;
    dev->read_func = ConsoleDevice::read;
    dev->block_func = ConsoleDevice::would_block;
    dev->inode = 0;
    dev->length = 0;
    dev->type = VFS_DEVICE;
    dev->flags = 0;
    dev->tty = 1;
    dev->uid = dev->gid = 0;
    dev->mode = 0666;
    dev->atime = dev->ctime = dev->mtime = clock_now();
    strncpy(dev->name, devname, sizeof(dev->name));
    return dev;
}

ssize_t ConsoleDevice::write(VFS::Node* node, size_t, size_t size, const char* buffer)
{
    if (!node) return -1;
    TextRenderer::write(buffer, size);
    return (ssize_t)size;
}

ssize_t ConsoleDevice::read(VFS::Node* node, size_t, size_t size, char* buffer)
{
    if (!node) return -1;
    if (!conin_buffer) return 0;
    if (size > conin_bufsize) size = conin_bufsize;
    memcpy(buffer, conin_buffer, size);
    memmove(conin_buffer, conin_buffer + size, conin_bufsize - size);
    conin_bufsize -= size;
    conin_buffer = (char*)krealloc(conin_buffer, conin_bufsize);
    return (ssize_t)size;
}

void ConsoleDevice::append(char c)
{
    conin_bufsize++;
    conin_buffer = (char*)krealloc(
        conin_buffer, conin_bufsize); // FIXME: We should probably not be calling realloc every time a key is pressed.
    conin_buffer[conin_bufsize - 1] = c;
}